
resource "aws_api_gateway_rest_api" "jsonapi" {
  name        = "JsonApiRestGateway"
  description = "HTTP request front API for JSON API rest persistance"
  minimum_compression_size = 4096
}

# Path /{type}
resource "aws_api_gateway_resource" "typePath" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  parent_id   = "${aws_api_gateway_rest_api.jsonapi.root_resource_id}"
  path_part   = "{type}"
}

# Path /{type}/{id}
resource "aws_api_gateway_resource" "typeIdPath" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  parent_id   = "${aws_api_gateway_resource.typePath.id}"
  path_part   = "{id}"
}

# GET on path /{type}
resource "aws_api_gateway_method" "typePathGet" {
  rest_api_id   = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id   = "${aws_api_gateway_resource.typePath.id}"
  http_method   = "GET"
  authorization = "NONE"
}

# POST on path /{type}
resource "aws_api_gateway_method" "typePathPost" {
  rest_api_id   = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id   = "${aws_api_gateway_resource.typePath.id}"
  http_method   = "POST"
  authorization = "NONE"
}

# GET on path /{type}/{id}
resource "aws_api_gateway_method" "typeIdPathGet" {
  rest_api_id   = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id   = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method   = "GET"
  authorization = "NONE"
}

# DELETE on path /{type}/{id}
resource "aws_api_gateway_method" "typeIdPathDelete" {
  rest_api_id   = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id   = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method   = "DELETE"
  authorization = "NONE"
}

# PATCH on path /{type}/{id}
resource "aws_api_gateway_method" "typeIdPathPatch" {
  rest_api_id   = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id   = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method   = "PATCH"
  authorization = "NONE"
}

# Setup Integration Request for GET on {type}
resource "aws_api_gateway_integration" "lambdaJsonTypeGet" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typePathGet.resource_id}"
  http_method = "${aws_api_gateway_method.typePathGet.http_method}"

  integration_http_method = "POST"
  type                    = "AWS"
  uri                     = "${aws_lambda_function.lambda-jsonapi.invoke_arn}"
  # Transforms the incoming XML request to JSON
  passthrough_behavior    = "WHEN_NO_TEMPLATES"
  request_templates {
    "application/json" = "${file("template.vm")}"
  }
}

# Setup Integration Request for POST on {type}
resource "aws_api_gateway_integration" "lambdaJsonTypePost" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typePathPost.resource_id}"
  http_method = "${aws_api_gateway_method.typePathPost.http_method}"

  integration_http_method = "POST"
  type                    = "AWS"
  uri                     = "${aws_lambda_function.lambda-jsonapi.invoke_arn}"
  # Transforms the incoming XML request to JSON
  passthrough_behavior    = "WHEN_NO_TEMPLATES"
  request_templates {
    "application/json" = "${file("template.vm")}"
  }
}

# Setup Integration Request for GET on {type}/{id}
resource "aws_api_gateway_integration" "lambdaJsonTypeIdGet" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathGet.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathGet.http_method}"

  integration_http_method = "POST"
  type                    = "AWS"
  uri                     = "${aws_lambda_function.lambda-jsonapi.invoke_arn}"
  # Transforms the incoming XML request to JSON
  passthrough_behavior    = "WHEN_NO_TEMPLATES"
  request_templates {
    "application/json" = "${file("template.vm")}"
  }
}

# Setup Integration Request for DELETE on {type}/{id}
resource "aws_api_gateway_integration" "lambdaJsonTypeIdDelete" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathDelete.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathDelete.http_method}"

  integration_http_method = "POST"
  type                    = "AWS"
  uri                     = "${aws_lambda_function.lambda-jsonapi.invoke_arn}"
  # Transforms the incoming XML request to JSON
  passthrough_behavior    = "WHEN_NO_TEMPLATES"
  request_templates {
    "application/json" = "${file("template.vm")}"
  }
}

# Setup Integration Request for PATCH on {type}/{id}
resource "aws_api_gateway_integration" "lambdaJsonTypeIdPatch" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathPatch.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathPatch.http_method}"

  integration_http_method = "POST"
  type                    = "AWS"
  uri                     = "${aws_lambda_function.lambda-jsonapi.invoke_arn}"
  # Transforms the incoming XML request to JSON
  passthrough_behavior    = "WHEN_NO_TEMPLATES"
  request_templates {
    "application/json" = "${file("template.vm")}"
  }
}

# Setup Integration Response for status code 200 and GET on {type}
resource "aws_api_gateway_method_response" "200TypeGet" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typePathGet.resource_id}"
  http_method = "${aws_api_gateway_method.typePathGet.http_method}"
  status_code = "200"
}

resource "aws_api_gateway_integration_response" "200TypeGetIntegrationResponse" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typePath.id}"
  http_method = "${aws_api_gateway_method.typePathGet.http_method}"
  status_code = "${aws_api_gateway_method_response.200TypeGet.status_code}"
}

# Setup Integration Response for status code 200 and POST on {type}
resource "aws_api_gateway_method_response" "200TypePost" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typePath.id}"
  http_method = "${aws_api_gateway_method.typePathPost.http_method}"
  status_code = "200"
}

resource "aws_api_gateway_integration_response" "200TypePostIntegrationResponse" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typePath.id}"
  http_method = "${aws_api_gateway_method.typePathPost.http_method}"
  status_code = "${aws_api_gateway_method_response.200TypePost.status_code}"
  response_templates = {
    "application/json" = ""
  }
}

# Setup Integration Response for status code 200 and GET on {type}/{id}
resource "aws_api_gateway_method_response" "200TypeIdGet" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathGet.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathGet.http_method}"
  status_code = "200"
}

resource "aws_api_gateway_integration_response" "200TypeIdGetIntegrationResponse" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method = "${aws_api_gateway_method.typeIdPathGet.http_method}"
  status_code = "${aws_api_gateway_method_response.200TypeIdGet.status_code}"
  response_templates = {
    "application/json" = ""
  }
}

# Setup Integration Response for status code 200 and DELETE on {type}/{id}
resource "aws_api_gateway_method_response" "200TypeIdDelete" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathDelete.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathDelete.http_method}"
  status_code = "200"
}

resource "aws_api_gateway_integration_response" "200TypeIdDeleteIntegrationResponse" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method = "${aws_api_gateway_method.typeIdPathDelete.http_method}"
  status_code = "${aws_api_gateway_method_response.200TypeIdDelete.status_code}"
  response_templates = {
    "application/json" = ""
  }
}

# Setup Integration Response for status code 200 and PATCH on {type}/{id}
resource "aws_api_gateway_method_response" "200TypeIdPatch" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_method.typeIdPathPatch.resource_id}"
  http_method = "${aws_api_gateway_method.typeIdPathPatch.http_method}"
  status_code = "200"
}

resource "aws_api_gateway_integration_response" "200TypeIdPatchIntegrationResponse" {
  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  resource_id = "${aws_api_gateway_resource.typeIdPath.id}"
  http_method = "${aws_api_gateway_method.typeIdPathPatch.http_method}"
  status_code = "${aws_api_gateway_method_response.200TypeIdPatch.status_code}"
  response_templates = {
    "application/json" = ""
  }
}

resource "aws_api_gateway_deployment" "jsonapiDeployment" {
  depends_on = [
    "aws_api_gateway_integration.lambdaJsonTypeGet",
    "aws_api_gateway_integration.lambdaJsonTypePost",
    "aws_api_gateway_integration.lambdaJsonTypeIdGet",
    "aws_api_gateway_integration.lambdaJsonTypeIdPatch",
    "aws_api_gateway_integration.lambdaJsonTypeIdDelete"
  ]

  rest_api_id = "${aws_api_gateway_rest_api.jsonapi.id}"
  stage_name  = "staging"
}

output "base_url" {
  value = "${aws_api_gateway_deployment.jsonapiDeployment.invoke_url}"
}